Package: extraChIPs
Version: 1.16.1
Title: Additional functions for working with ChIP-Seq data
Authors@R: person("Stevie", "Pederson", 
  email = "stephen.pederson.au@gmail.com", 
  role = c("aut", "cre"),
  comment = c(ORCID = "0000-0001-8197-3303")
  )
Description: This package builds on existing tools and adds some simple
        but extremely useful capabilities for working wth ChIP-Seq
        data. The focus is on detecting differential binding
        windows/regions. One set of functions focusses on
        set-operations retaining mcols for GRanges objects, whilst
        another group of functions are to aid visualisation of results.
        Coercion to tibble objects is also implemented.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/smped/extraChIPs
BugReports: https://github.com/smped/extraChIPs/issues
Depends: BiocParallel, R (>= 4.2.0), GenomicRanges, ggplot2 (>= 4.0.0),
        ggside (>= 0.4.0), Seqinfo, SummarizedExperiment (>= 1.39.1),
        tibble
Imports: csaw, dplyr (>= 1.1.1), edgeR (>= 4.0), forcats, GenomeInfoDb,
        glue, ggrepel, InteractionSet, IRanges, matrixStats, methods,
        patchwork, RColorBrewer, rlang, Rsamtools, rtracklayer,
        S4Vectors, scales, stats, stringr, tidyr, tidyselect, vctrs
Suggests: apeglm, BiocStyle, SimpleUpset, covr, DESeq2,
        EnrichedHeatmap, GenomicAlignments, GenomicInteractions, Gviz,
        ggforce, harmonicmeanp, here, knitr, limma, magrittr,
        plyranges, quantro, rmarkdown, testthat (>= 3.0.0), tidyverse,
        VennDiagram
biocViews: ChIPSeq, HiC, Sequencing, Coverage
BiocType: Software
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.3
Config/testthat/edition: 3
Config/pak/sysreqs: make libbz2-dev libicu-dev liblzma-dev libxml2-dev
        libssl-dev xz-utils zlib1g-dev
Repository: https://bioc-release.r-universe.dev
Date/Publication: 2026-05-12 16:36:09 UTC
RemoteUrl: https://github.com/bioc/extraChIPs
RemoteRef: RELEASE_3_23
RemoteSha: b547f268263da33d9a14ad4714b2eb60e3948cfe
NeedsCompilation: yes
Packaged: 2026-05-12 21:22:56 UTC; root
Author: Stevie Pederson [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8197-3303>)
Maintainer: Stevie Pederson <stephen.pederson.au@gmail.com>
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-12 21:25:17 UTC; unix
