# Generated by roxygen2: do not edit by hand

S3method(as_tibble,DataFrame)
S3method(as_tibble,GInteractions)
S3method(as_tibble,GenomicRanges)
S3method(as_tibble,Seqinfo)
S3method(as_tibble,SummarizedExperiment)
S3method(as_tibble,TopTags)
export(addDiffStatus)
export(bestOverlap)
export(centrePeaks)
export(chopMC)
export(colToRanges)
export(collapseGenes)
export(defineRegions)
export(defineSeqinfo)
export(distinctMC)
export(dualFilter)
export(fitAssayDiff)
export(getProfileData)
export(grlToSE)
export(importPeaks)
export(intersectMC)
export(makeConsensus)
export(mapByFeature)
export(mapGrlCols)
export(mergeByCol)
export(mergeByHMP)
export(mergeBySig)
export(partitionRanges)
export(plotAssayDensities)
export(plotAssayHeatmap)
export(plotAssayPCA)
export(plotAssayRle)
export(plotGrlCol)
export(plotHFGC)
export(plotOverlaps)
export(plotPairwise)
export(plotPie)
export(plotProfileHeatmap)
export(plotSplitDonut)
export(propOverlap)
export(reduceMC)
export(setdiffMC)
export(stitchRanges)
export(unionMC)
export(voomWeightsFromCPM)
exportMethods(addDiffStatus)
exportMethods(bestOverlap)
exportMethods(centrePeaks)
exportMethods(colToRanges)
exportMethods(fitAssayDiff)
exportMethods(getProfileData)
exportMethods(grlToSE)
exportMethods(intersectMC)
exportMethods(mergeByCol)
exportMethods(mergeByHMP)
exportMethods(mergeBySig)
exportMethods(partitionRanges)
exportMethods(plotAssayDensities)
exportMethods(plotAssayHeatmap)
exportMethods(plotAssayPCA)
exportMethods(plotAssayRle)
exportMethods(plotOverlaps)
exportMethods(plotPie)
exportMethods(plotProfileHeatmap)
exportMethods(plotSplitDonut)
exportMethods(propOverlap)
exportMethods(setdiffMC)
exportMethods(unionMC)
import(GenomicRanges)
import(SummarizedExperiment)
import(ggplot2)
import(ggside)
import(methods)
importClassesFrom(GenomicRanges,GRanges)
importClassesFrom(IRanges,CompressedList)
importClassesFrom(IRanges,RleList)
importClassesFrom(IRanges,RleViewsList)
importClassesFrom(Rsamtools,BamFileList)
importClassesFrom(S4Vectors,DataFrame)
importClassesFrom(S4Vectors,HitsList)
importClassesFrom(SummarizedExperiment,RangedSummarizedExperiment)
importClassesFrom(rtracklayer,BigWigFile)
importClassesFrom(rtracklayer,BigWigFileList)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bpisup)
importFrom(BiocParallel,bplapply)
importFrom(BiocParallel,bpparam)
importFrom(BiocParallel,bpstart)
importFrom(BiocParallel,bpstop)
importFrom(GenomeInfoDb,keepSeqlevels)
importFrom(GenomicRanges,setdiff)
importFrom(IRanges,RleList)
importFrom(IRanges,RleViewsList)
importFrom(IRanges,SplitDataFrameList)
importFrom(IRanges,Views)
importFrom(IRanges,commonColnames)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,subsetByOverlaps)
importFrom(IRanges,viewMaxs)
importFrom(IRanges,viewWhichMaxs)
importFrom(InteractionSet,anchors)
importFrom(RColorBrewer,brewer.pal)
importFrom(Rsamtools,BamFileList)
importFrom(Rsamtools,ScanBamParam)
importFrom(Rsamtools,countBam)
importFrom(Rsamtools,idxstatsBam)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,"metadata<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,Rle)
importFrom(S4Vectors,SimpleList)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,metadata)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(S4Vectors,subset)
importFrom(Seqinfo,"seqinfo<-")
importFrom(Seqinfo,Seqinfo)
importFrom(Seqinfo,genome)
importFrom(Seqinfo,seqinfo)
importFrom(Seqinfo,seqlevels)
importFrom(Seqinfo,seqnames)
importFrom(csaw,combineTests)
importFrom(csaw,filterWindowsControl)
importFrom(csaw,filterWindowsProportion)
importFrom(csaw,getBestTest)
importFrom(csaw,minimalTests)
importFrom(csaw,readParam)
importFrom(csaw,scaleControlFilter)
importFrom(csaw,windowCounts)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,bind_cols)
importFrom(dplyr,bind_rows)
importFrom(dplyr,case_when)
importFrom(dplyr,desc)
importFrom(dplyr,distinct)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,inner_join)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,mutate_all)
importFrom(dplyr,n)
importFrom(dplyr,pick)
importFrom(dplyr,select)
importFrom(dplyr,summarise)
importFrom(dplyr,ungroup)
importFrom(edgeR,DGEList)
importFrom(edgeR,cpm)
importFrom(edgeR,estimateDisp)
importFrom(edgeR,glmQLFTest)
importFrom(edgeR,glmQLFit)
importFrom(edgeR,glmTreat)
importFrom(edgeR,normLibSizes)
importFrom(edgeR,topTags)
importFrom(forcats,fct_cross)
importFrom(forcats,fct_explicit_na)
importFrom(forcats,fct_inorder)
importFrom(forcats,fct_na_value_to_level)
importFrom(forcats,fct_relabel)
importFrom(forcats,fct_rev)
importFrom(ggrepel,geom_label_repel)
importFrom(ggrepel,geom_text_repel)
importFrom(glue,glue)
importFrom(glue,glue_collapse)
importFrom(matrixStats,rowMeans2)
importFrom(matrixStats,rowMedians)
importFrom(matrixStats,rowSds)
importFrom(matrixStats,weightedMean)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,new)
importFrom(methods,slot)
importFrom(methods,slotNames)
importFrom(patchwork,area)
importFrom(patchwork,plot_layout)
importFrom(patchwork,plot_spacer)
importFrom(rlang,"!!!")
importFrom(rlang,"!!")
importFrom(rlang,":=")
importFrom(rlang,.data)
importFrom(rlang,as_label)
importFrom(rlang,enexpr)
importFrom(rlang,enquo)
importFrom(rlang,ensym)
importFrom(rlang,eval_tidy)
importFrom(rlang,list2)
importFrom(rlang,quo_is_null)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(rlang,syms)
importFrom(rtracklayer,BigWigFile)
importFrom(rtracklayer,BigWigFileList)
importFrom(rtracklayer,import.bw)
importFrom(rtracklayer,ucscGenomes)
importFrom(scales,comma)
importFrom(scales,percent)
importFrom(stats,approxfun)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,density)
importFrom(stats,lowess)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,p.adjust.methods)
importFrom(stats,prcomp)
importFrom(stats,quantile)
importFrom(stats,setNames)
importFrom(stats,weighted.mean)
importFrom(stringr,str_count)
importFrom(stringr,str_pad)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_replace_na)
importFrom(stringr,str_sort)
importFrom(stringr,str_to_title)
importFrom(stringr,str_trim)
importFrom(stringr,str_trunc)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,all_of)
importFrom(tidyr,chop)
importFrom(tidyr,complete)
importFrom(tidyr,nest)
importFrom(tidyr,pivot_longer)
importFrom(tidyr,pivot_wider)
importFrom(tidyr,unite)
importFrom(tidyr,unnest)
importFrom(tidyselect,all_of)
importFrom(tidyselect,ends_with)
importFrom(tidyselect,everything)
importFrom(vctrs,vec_proxy)
importMethodsFrom(SummarizedExperiment,'assay<-')
importMethodsFrom(SummarizedExperiment,'colData<-')
importMethodsFrom(SummarizedExperiment,'rowData<-')
importMethodsFrom(SummarizedExperiment,assay)
importMethodsFrom(SummarizedExperiment,colData)
importMethodsFrom(SummarizedExperiment,rowData)
importMethodsFrom(SummarizedExperiment,rowRanges)
useDynLib(extraChIPs, .registration = TRUE)
