# Currently set for avoidance of NOTE msesages using R CMD check
# As code is changed, these may be removed
globalVariables(
    c(
      "count", "d", "i", "lab", "name", "N", "p", "r", "range", "value",
      "x", "y"
    )
)
